//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "sockets.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "searchlist.h"
#include <time.h>
#include <afxmt.h>
#include <afxinet.h>
#include "UDPSocket.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// CServerConnect

void CServerConnect::TryAnotherConnectionrequest()
{
  EMULE_TRY
	if ( connectionattemps.GetCount()< ((app_prefs->IsSafeConnectEnabled()) ? 1 : 2)) {

		CServer*  next_server = used_list->GetNextServer();

		if (!next_server)
		{
			if (connectionattemps.GetCount()==0){
				AddLogLine(true,GetResString(IDS_OUTOFSERVERS));
				ConnectToAnyServer(lastStartAt);
			}
			return;
		}

		// Barry - Only auto-connect to static server option
		if (theApp.glob_prefs->AutoConnectStaticOnly())
		{
			if (next_server->IsStaticMember())
                ConnectToServer(next_server,true);
		}
		else
			ConnectToServer(next_server,true);
	}
  EMULE_CATCH
}

void CServerConnect::ConnectToAnyServer(uint32 startAt,bool prioSort,bool isAuto)
{
  EMULE_TRY
	lastStartAt=startAt;
	StopConnectionTry();
	Disconnect();
	theApp.emuledlg->ShowConnectionState(false,"",true);
	connecting = true;
	singleconnecting = false;

	// Barry - Only auto-connect to static server option
	if (theApp.glob_prefs->AutoConnectStaticOnly() && isAuto)
	{
		bool anystatic = false;
		CServer *next_server; 
		used_list->SetServerPosition( startAt );
		while ((next_server = used_list->GetNextServer()) != NULL)
		{
			if (next_server->IsStaticMember())
			{
				anystatic = true;
				break;
			}
		}
		if (!anystatic)
		{
			connecting = false;
			AddLogLine(true,GetResString(IDS_ERR_NOVALIDSERVERSFOUND));
			return;
		}
	}

	used_list->SetServerPosition( startAt );
	if( theApp.glob_prefs->Score() && prioSort ) used_list->Sort();

	if (used_list->GetServerCount()==0 ){
		connecting = false;
		AddLogLine(true,GetResString(IDS_ERR_NOVALIDSERVERSFOUND));
		return;
	}
	theApp.listensocket->Process();

	TryAnotherConnectionrequest();
  EMULE_CATCH
}

void CServerConnect::ConnectToServer(CServer* server, bool multiconnect){
  EMULE_TRY
	if (!multiconnect) {
		StopConnectionTry();
		Disconnect();
	}
	connecting = true;
	singleconnecting = !multiconnect;

	CServerSocket* newsocket = new CServerSocket(this);
	m_lstOpenSockets.AddTail((void*&)newsocket);
	newsocket->Create(0,SOCK_STREAM,FD_READ|FD_WRITE|FD_CLOSE|FD_CONNECT,NULL);
	newsocket->ConnectToServer(server);
	ULONG x=GetTickCount();
	connectionattemps.SetAt(x,newsocket);
  EMULE_CATCH
}

void CServerConnect::StopConnectionTry(){
  EMULE_TRY
	connectionattemps.RemoveAll();
	connecting = false;
	singleconnecting = false;

	if (m_idRetryTimer) 
	{ 
		KillTimer(NULL, m_idRetryTimer); 
		m_idRetryTimer= 0; 
	} 

	// close all currenty opened sockets except the one which is connected to our current server
	for( POSITION pos = m_lstOpenSockets.GetHeadPosition(); pos != NULL; )
	{
		CServerSocket* pSck = (CServerSocket*)m_lstOpenSockets.GetNext(pos);
		if (pSck == connectedsocket)		// don't destroy socket which is connected to server
			continue;
		if (pSck->m_bIsDeleting == false)	// don't destroy socket if it is going to destroy itself later on
			DestroySocket(pSck);
	}
  EMULE_CATCH
}

void CServerConnect::ConnectionEstablished(CServerSocket* sender)
{
  EMULE_TRY
	m_nConnectionAttempts = 0;
	if(m_idCheckTimer)
	  	KillTimer(NULL, m_idCheckTimer);
	m_idCheckTimer = 0;

	if (connecting == false)
	{
		// we are already connected to another server
		DestroySocket(sender);
		return;
	}
	if (sender->GetConnectionState() == CS_WAITFORLOGIN){
		//Cax2 - reset the timeout timer
		ULONG tmpkey;
		CServerSocket* tmpsock;
		POSITION pos = connectionattemps.GetStartPosition();
		while (pos)
		{
			connectionattemps.GetNextAssoc(pos,tmpkey,tmpsock);
			if (tmpsock==sender)
			{
				connectionattemps.RemoveKey(tmpkey);
				tmpkey=GetTickCount();
				connectionattemps.SetAt(tmpkey,sender);
				break;
			}
		}
		//Cax2 - end reset the timeout timer
		AddLogLine(false,GetResString(IDS_CONNECTEDTOREQ),sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort());
		//send loginpacket
		CServer* update = theApp.serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
		if (update){
			update->ResetFailedCount();
			theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer( update );
		}
		CMemFile* data = new CMemFile();
		data->Write(theApp.glob_prefs->GetUserHash(),16);
		uint32 clientid = GetClientID();
		data->Write(&clientid,4);
		uint16 port = app_prefs->GetPort();
		data->Write(&port,2);
		uint32 tagcount = 3;
		data->Write(&tagcount,4);
		CTag* tag = new CTag(CT_NAME,app_prefs->GetUserNick().GetBuffer());
		tag->WriteTagToFile(data);
		delete tag;
		tag = new CTag(CT_VERSION,EDONKEYVERSION);
		tag->WriteTagToFile(data);
		delete tag;
		tag = new CTag(CT_PORT,app_prefs->GetPort());
		tag->WriteTagToFile(data);
		delete tag;

		Packet* packet = new Packet(data);
		packet->opcode = OP_LOGINREQUEST;
		theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
		this->SendPacket(packet,true,sender);
		delete data;
	}
	else if (sender->GetConnectionState() == CS_CONNECTED){	
		theApp.stat_reconnects++;
		theApp.stat_serverConnectTime=GetTickCount();
		connected = true;
		AddLogLine(true,GetResString(IDS_CONNECTEDTO),sender->cur_server->GetListName());
		theApp.emuledlg->ShowConnectionState(true,sender->cur_server->GetListName());
		connectedsocket = sender;
		StopConnectionTry();
		theApp.sharedfiles->SendListToServer();
		theApp.emuledlg->serverwnd.serverlistctrl.RemoveDeadServer();
		// tecxx 1609 2002 - serverlist update
		if (theApp.glob_prefs->GetAddServersFromConn())
		{
			Packet* packet = new Packet(OP_GETSERVERLIST,0);
			theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
			SendPacket(packet,true);
		}
	}
  EMULE_CATCH
}

bool CServerConnect::SendPacket(Packet* packet,bool delpacket, CServerSocket* to){
	if (!to){
		if (connected){
			connectedsocket->SendPacket(packet,delpacket,true);
		}
		else
			return false;
	}
	else{
		to->SendPacket(packet,delpacket,true);
	}
	return true;
}

bool CServerConnect::SendUDPPacket(Packet* packet,CServer* host,bool delpacket){
		if (connected){
			udpsocket->SendPacket(packet,host);
		}
		if (delpacket)
			delete packet;
	return true;
}

void CServerConnect::ConnectionFailed(CServerSocket* sender){
	if (connecting == false && sender != connectedsocket)
	{
		// just return, cleanup is done by the socket itself
		return;
	}
	//messages + fail tries count
	CServer* update;
	switch (sender->GetConnectionState()){
		case CS_FATALERROR:
			AddLogLine(true,GetResString(IDS_ERR_FATAL));
			m_nConnectionAttempts++;
			break;
		case CS_DISCONNECTED:
			AddLogLine(true,GetResString(IDS_ERR_LOSTC),sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort());
			break;
		case CS_SERVERDEAD:
			AddLogLine(false,GetResString(IDS_ERR_DEAD),sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort()); //<<--
			update = theApp.serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
			if(update){
				update->AddFailedCount();
				theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer( update );
			}
			m_nConnectionAttempts++;
			break;
		case CS_ERROR:
			m_nConnectionAttempts++;
			break;
		case CS_SERVERFULL:
			AddLogLine(false,GetResString(IDS_ERR_FULL),sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort());
			break;
		case CS_NOTCONNECTED: 
			m_nConnectionAttempts++;
			break; 
	}

	// IMPORTANT: mark this socket not to be deleted in StopConnectionTry(), 
	// because it will delete itself after this function!
	sender->m_bIsDeleting = true;

	if(m_nConnectionAttempts >= 3)
	{
		connected = false;
		if (connectedsocket) 
			connectedsocket->Close();
		connectedsocket = NULL;
		AddLogLine(false, GetResString(IDS_NET_CHECKING));
		theApp.emuledlg->searchwnd.OnBnClickedCancels();
		theApp.emuledlg->ShowConnectionState(false,"",true);
		StopConnectionTry();
		if(m_idCheckTimer)
			KillTimer(NULL, m_idCheckTimer);
		m_idCheckTimer = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, (TIMERPROC)CheckInternetCallback);
		return;
	}

	switch (sender->GetConnectionState()){
		case CS_FATALERROR:{
			bool autoretry= !singleconnecting;
			StopConnectionTry();
			if ((app_prefs->Reconnect()) && (autoretry) && (!m_idRetryTimer)){ 
				AddLogLine(false,GetResString(IDS_RECONNECT), CS_RETRYCONNECTTIME); 
				m_idRetryTimer = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, (TIMERPROC)RetryConnectCallback);
			}
			theApp.emuledlg->ShowConnectionState(false,"",true);
			break;
		}
		case CS_DISCONNECTED:{
			connected = false;
			if (connectedsocket) 
				connectedsocket->Close();
			connectedsocket = NULL;
			theApp.emuledlg->searchwnd.OnBnClickedCancels();
			if (app_prefs->Reconnect() && !connecting){
				ConnectToAnyServer();		
			}
			if (theApp.glob_prefs->GetNotifierPopOnImportantError()) {
				theApp.emuledlg->ShowNotifier(GetResString(IDS_CONNECTIONLOST), TBN_IMPORTANTEVENT, false);
			}
			theApp.emuledlg->ShowConnectionState(false,"",true);
			break;
		}
		case CS_ERROR:
		case CS_NOTCONNECTED:{
			if (!connecting)
				break;
			AddLogLine(false,GetResString(IDS_ERR_CONFAILED),sender->info, sender->cur_server->GetFullIP(), sender->cur_server->GetPort() );
		}
		case CS_SERVERDEAD:
		case CS_SERVERFULL:{
			if (!connecting)
				break;
			if (singleconnecting){
				StopConnectionTry();
				break;
			}

			DWORD tmpkey;
			CServerSocket* tmpsock;
			POSITION pos = connectionattemps.GetStartPosition();
			while (pos){
				connectionattemps.GetNextAssoc(pos,tmpkey,tmpsock);
				if (tmpsock==sender) {
					connectionattemps.RemoveKey(tmpkey);
					break;
				}
			}			
			TryAnotherConnectionrequest();
		}
	}
	theApp.emuledlg->ShowConnectionState(false,"",true);
}

// 09/28/02, by zegzav
VOID CALLBACK CServerConnect::RetryConnectCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime) 
{ 
	CServerConnect *_this= theApp.serverconnect; 
	ASSERT(_this); 
	_this->StopConnectionTry();
	if (_this->IsConnected()) return; 

	_this->ConnectToAnyServer(); 
}

static TCHAR g_Urls[4][50] = {
	/*_T("www.microsoft.com"),
	_T("www.yahoo.com"),
	_T("www.ibm.com"),*/
	_T("www.army.mil"),
	_T("www.networksolutions.com"),
	_T("www.ripe.net"),
	_T("www.w3.org")
};

VOID CALLBACK CServerConnect::CheckInternetCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime) 
{ 
	CServerConnect *_this= theApp.serverconnect; 
	ASSERT(_this); 

	KillTimer(NULL, _this->m_idCheckTimer);
	_this->m_idCheckTimer = 0;

	AddLogLine(false, GetResString(IDS_NET_CHECKING));

	if(_this->IsConnected()) return;

	if(_this->IsConnecting())
		_this->StopConnectionTry();

	CString sUrl;
	sUrl.Format("http://%s", g_Urls[rand() % 4]);
	if(_this->IsAliveURL(sUrl))
	{
		AddLogLine(false,GetResString(IDS_NET_WORKING));
		_this->ConnectToAnyServer(); 
	}
	else
	{
		_this->m_idCheckTimer = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, (TIMERPROC)CheckInternetCallback);
	}
}

bool CServerConnect::IsAliveURL(LPCSTR sURL)
{
	HINTERNET hOpen = InternetOpen(_T(""), INTERNET_OPEN_TYPE_PRECONFIG , NULL, NULL, 0);
	if(!hOpen)
		return false;
 
	HINTERNET hUrl =  InternetOpenUrl(hOpen, sURL, _T(""), NULL, /*INTERNET_FLAG_NO_AUTH | */INTERNET_FLAG_NO_CACHE_WRITE, NULL);
	if(!hUrl)
	{
		InternetCloseHandle(hOpen);
		return false;
	}
 
	TCHAR szBuffer[100];
	DWORD dwBytes = 100;
	DWORD dwIndex = 0;
	BOOL bQ = HttpQueryInfo(hUrl, HTTP_QUERY_STATUS_CODE, szBuffer, &dwBytes, &dwIndex);

	bool bSuccess = bQ && !strcmp(szBuffer, _T("200"));

	InternetCloseHandle(hUrl);
	InternetCloseHandle(hOpen);
 
	return bSuccess;
}

void CServerConnect::CheckForTimeout()
{ 
	//AddDebugLogLine(false,"checking timeout: %u",CONSERVTIMEOUT/6000 );
	DWORD maxage=GetTickCount() - CONSERVTIMEOUT;
	DWORD tmpkey;
	CServerSocket* tmpsock;
	POSITION pos = connectionattemps.GetStartPosition();
	while (pos){
		connectionattemps.GetNextAssoc(pos,tmpkey,tmpsock);
		//AddDebugLogLine(false,"tmpkey: %u - maxage: %u",tmpkey ,maxage);
		if (!tmpsock) {
			AddDebugLogLine(false,"Error: Socket invalid at timeoutcheck" );
			connectionattemps.RemoveKey(tmpkey);
			return;
		}

		if (tmpkey<=maxage) {
			AddLogLine(false,GetResString(IDS_ERR_CONTIMEOUT),tmpsock->info, tmpsock->cur_server->GetFullIP(), tmpsock->cur_server->GetPort() );
			connectionattemps.RemoveKey(tmpkey);
			TryAnotherConnectionrequest();
			DestroySocket(tmpsock);
		}
	}
}

bool CServerConnect::Disconnect(){
	if (connected && connectedsocket){
		DestroySocket(connectedsocket);
		connectedsocket = NULL;
		connected = false;
		theApp.emuledlg->ShowConnectionState(false,"");
		theApp.emuledlg->serverwnd.servermsgbox.AppendText(CString("\n\n\n\n"));
		theApp.stat_serverConnectTime=0;
		return true;
	}
	else
		return false;
}

CServerConnect::CServerConnect(CServerList* in_serverlist, CPreferences* in_prefs){
	connectedsocket = NULL;
	app_prefs = in_prefs;
	used_list = in_serverlist;
	max_simcons = (app_prefs->IsSafeConnectEnabled()) ? 1 : 2;
	connecting = false;
	connected = false;
	clientid = 0;
	singleconnecting = false;
	udpsocket = new CUDPSocket(this); // initalize socket for udp packets
	udpsocket->Create();
	m_idRetryTimer= 0;
	lastStartAt=0;
	m_idCheckTimer = 0;
	m_nConnectionAttempts = 0;
}

CServerConnect::~CServerConnect(){
	// stop all connections
	StopConnectionTry();
	// close connected socket, if any
	DestroySocket(connectedsocket);
	connectedsocket = NULL;
	// close udp socket
	udpsocket->Close();
	delete udpsocket;
}

CServer* CServerConnect::GetCurrentServer(){
	if (IsConnected() && connectedsocket)
		return connectedsocket->cur_server;
	return NULL;
}

void CServerConnect::SetClientID(uint32 newid){
	clientid = newid;
	theApp.emuledlg->ShowConnectionState(IsConnected(),GetCurrentServer()->GetListName() );
}

void CServerConnect::DestroySocket(CServerSocket* pSck){
	if (pSck == NULL)
		return;
	// remove socket from list of opened sockets
	for( POSITION pos = m_lstOpenSockets.GetHeadPosition(); pos != NULL; )
	{
		POSITION posDel = pos;
		CServerSocket* pTestSck = (CServerSocket*)m_lstOpenSockets.GetNext(pos);
		if (pTestSck == pSck)
		{
			m_lstOpenSockets.RemoveAt(posDel);
			break;
		}
	}
	// changed to AsyncSocketEx - deadlake / Maverick
	if (pSck->m_SocketData.hSocket != INVALID_SOCKET){
		pSck->AsyncSelect(0);
		pSck->Close();
	}

	delete pSck;
}

bool CServerConnect::IsLocalServer(uint32 dwIP, uint16 nPort){
	if (IsConnected()){
		if (connectedsocket->cur_server->GetIP() == dwIP && connectedsocket->cur_server->GetPort() == nPort)
			return true;
	}
	return false;
}